#pragma rtGlobals = 1		// modern global method

#include <Keyword-Value>
#include <Strings as Lists>
#include <Readback ModifyStr>

// PPT Control Panel - XChart Graph Control
// part of the PPT, the "Patcher's Power Tools"
//  24.10.96 Francisco Mendez (based on J. Biedermann's Control Panel)
//
// 29.10.96 added window hook to update controls on activate event
// 30.11.96 graph window will be sized to fit the control bar
// 01.12.96 axis-Popup displays waves controlling the axis instead of the axisnames
// 04.12.96 bottom-/top-axis is identified by type rather than by the string "bottom"
// 07.12.96 target  window will be brought to front before opening the panel
// 07.12.96 control panel will only be opened, if it doesn't exist already
// 12.01.97 SetVariables are no more "Chicago" (problems with system font replacement)
// 23.10.97 macro stores PPT-globals in folder "PPTGlobals", Igor Globals are not created

// ****************************************************

Menu "Macros"
	"Graph Control Bar/3", PPTGraphControl ()
End

// ****************************************************

Function PPTPanelGlobals ()
	Silent 1							// initializing . . .

	String oldDataFolder = GetDataFolder (1)
	if (! DataFolderExists ("root:PPTGlobals"))
		NewDataFolder /O root:PPTGlobals
	endif

	SetDataFolder root:PPTGlobals	// create the PPT global variables

	Variable /G	pptMinVal			// minimum of axis
	Variable /G	pptMaxVal			// maximum of axis
	Variable /G	pptDelta = 10	// 10% movement
	Variable /G	pptAxisPos		// position of axis [%]
	Variable /G pptAxisSize		// size of axis [%]
	Variable /G	pptShowAxes = 0	// show waves instead of axes

	SetDataFolder $oldDataFolder
End Function

// ****************************************************

Function PPTGraphControl () : Panel
	//Silent 1; PauseUpdate

	Variable	barW = 426			// width of control panel
	Variable	barH = 90				// height of control panel
	String		topGraph				// name of target graph
	String		S_Value

	topGraph = WinName (0, 1)		// get name of the target graph
	if (strlen (topGraph) <= 0)
		Abort "The control panel will be installed into the top graph, but there aren't any graphs!"
	endif

	GetWindow $topGraph hook
	if (cmpstr (S_Value, "PPTControlUpdateHook") == 0)
		Beep							// control panel already in window
		Abort "There's already a control panel in the window!"
	endif
	DoWindow /F $topGraph			// bring graph to the front

	SetWindow $topGraph hook = PPTControlUpdateHook
	GetWindow $topGraph, wsize	// resize graph to fit the control bar
	if ((V_right - V_left) < barW)
		MoveWindow V_left, V_top, V_left + barW, V_bottom
	endif

	PPTPanelGlobals ()				// create globals
	ControlBar barH
	Execute "ModifyGraph cbRGB = (56797,56797,56797)"

	PopupMenu pChooseAxis	proc = PPTChooseAxis
	PopupMenu pChooseAxis	value =  PPTGetAxesWaves ()
	PopupMenu pChooseAxis	pos = {6,2}, size = {82,19}
	PopupMenu pChooseAxis	mode = 1
	PopupMenu pChooseAxis	help = {"Sets the current axis or controlling wave for modification."}
	PPTSetAxisPopupTitle ()

	SetVariable tMove		proc = PPTSetDelta
	SetVariable tMove		title = "move in %"
	SetVariable tMove		value = root:PPTGlobals:pptDelta
	SetVariable tMove		limits = {0,INF,5}
	SetVariable tMove		pos = {6,25}, size = {134,17}
	SetVariable tMove		help = {"Sets amount to shift axes or in-/decrease bounds."}

	Button bShiftLeft		proc = PPTShiftAxis
	Button bShiftLeft		title = "<"
	Button bShiftLeft		pos = {6,49}, size = {30,30}
	Button bShiftLeft		help = {"Decreases lower and upper margin."} 

	SetVariable tMinValue	proc = PPTSetMinMax
	SetVariable tMinValue	title = " "
	SetVariable tMinValue	value = root:PPTGlobals:pptMinVal
	SetVariable tMinValue	pos = {44, 46}, size = {96, 17}
	SetVariable tMinValue	help = {"Sets lower limit of current axis. Type new value or use arrows to change it."}

	SetVariable tMaxValue	proc = PPTSetMinMax
	SetVariable tMaxValue	title = " "
	SetVariable tMaxValue	value = root:PPTGlobals:pptMaxVal
	SetVariable tMaxValue	pos = {44, 64}, size = {96, 17}
	SetVariable tMaxValue	help = {"Sets upper limit of current axis. Type new value or use arrows to change it."}

	Button bShiftRight		proc = PPTShiftAxis
	Button bShiftRight		title = ">"
	Button bShiftRight		pos = {144, 49}, size = {30, 30}
	Button bShiftRight		help = {"Increases lower and upper margin."}

	CheckBox cShowAxes	proc = PPTUpdateAxesPopup
	CheckBox cShowAxes	pos = {178, 2}, size = {90, 20}
	CheckBox cShowAxes	title = "Show Axes"
	CheckBox cShowAxes	help = {"If checked, the axis-popup will show the axis names, otherwise the controlling waves will be displayed."}

	Button bAutoScale		proc = PPTAutoScaleAxis
	Button bAutoScale		title = "AutoScale Axis"
	Button bAutoScale		pos = {178, 24}, size = {112, 20}
	Button bAutoScale		help = {"AutoScales the current axis."}

	Button bAutoScale_all	proc = PPTAutoScaleAll
	Button bAutoScale_all	title = "AutoScale all"
	Button bAutoScale_all	pos = {178, 46}, size = {112, 20}
	Button bAutoScale_all	help = {"Identical to the Graph's menu item \"AutoScale Axes\"."}

	Button bSetAllY			proc = PPTSetAllAxes
	Button bSetAllY			title = "Set all"
	Button bSetAllY			pos = {178, 68}, size = {112, 20}
	Button bSetAllY			help = {"Sets all y-Axes to the values defined by lower and upper limit."}	

	Button bClose			proc = PPTCloseControl
	Button bClose			title = "Close Panel"
	Button bClose			pos = {310, 2}, size = {112, 30}
	Button bClose			help = {"Removes control bar from graph."}	

	SetVariable tDrawPos	proc = PPTSetAxisMinMax
	SetVariable tDrawPos	title = "Draw at %"
	SetVariable tDrawPos	pos = {293, 46}, size = {128,17}
	SetVariable tDrawPos	value = root:PPTGlobals:pptAxisPos
	SetVariable tDrawPos	limits = {0,100,1}
	SetVariable tDrawPos	help = {"Defines the position at which the current axis has to be drawn."}

	SetVariable tDrawSize	proc = PPTSetAxisMinMax
	SetVariable tDrawSize	title = "size %"
	SetVariable tDrawSize	pos = {318, 64}, size = {103, 17}
	SetVariable tDrawSize	value = root:PPTGlobals:pptAxisSize
	SetVariable tDrawSize	limits = {0, 100, 1}
	SetVariable tDrawSize	help = {"Defines the size of the current axis."}

	Execute "PPTSetPanelCheckboxes ()"
	PPTUpdateControlPanel ()
End Window

// ****************************************************

Proc PPTSetPanelCheckboxes ()		// Doesn't work within functions
	CheckBox cShowAxes value = root:PPTGlobals:pptShowAxes
End Proc

// ****************************************************
// returns a list of all waves controlling the graph axes, sorted by axisType
// ****************************************************

Function /S PPTGetAxesWaves ()
	String		theAxisList, theAxisName
	String		theLeftList, theRightList, theBottomList, theTopList
	String		theAxisWave, theAxisType, theAxisInfo
	String		theList
	Variable	i, len

	NVAR showAxes	= root:PPTGlobals:pptShowAxes

	theAxisList = AxisList ("")
	if (showAxes)
		return (theAxisList)
	endif

	i = 0
	theLeftList = ""
	theRightList = ""
	theBottomList = ""
	theTopList = ""

	do
		theAxisName = GetStrFromList (theAxisList, i, ";")
		if (strlen (theAxisName) == 0)					// end of list found
			break
		endif
		theAxisInfo = AxisInfo ("", theAxisName)
		theAxisWave = StrByKey ("CWAVE", theAxisInfo)
		theAxisType = StrByKey ("AXTYPE", theAxisInfo)

		if (cmpstr (theAxisType, "left") == 0)
			theLeftList += (theAxisWave + ";")
		endif
		if (cmpstr (theAxisType, "right") == 0)
			theRightList += (theAxisWave + ";")
		endif
		if (cmpstr (theAxisType, "top") == 0)
			theTopList += (theAxisWave + ";")
		endif
		if (cmpstr (theAxisType, "bottom") == 0)
			theBottomList += (theAxisWave + ";")
		endif

		i += 1
	while (1)

	theList = ""
	if (strlen (theLeftList) > 0)
		theList += (theLeftList + "-;")
	endif
	if (strlen (theRightList) > 0)
		theList += (theRightList + "-;")
	endif
	if (strlen (theTopList) > 0)
		theList += (theTopList + "-;")
	endif
	if (strlen (theBottomList) > 0)
		theList += (theBottomList + "-;")
	endif

	len = strlen (theList)
	if (len > 0)
		theList [len -2, len - 1] = ""
	endif

	return (theList)
End Function

// ****************************************************
// window update hook
// ****************************************************

Function PPTControlUpdateHook (infoStr)
	String infoStr

	String eventKey = StrByKey ("EVENT", infoStr)
	if (cmpstr (eventKey, "activate") == 0)
		PPTUpdateControlPanel ()
	endif
	if (cmpstr (eventKey, "deactivate") == 0)
		// Handle Deactivate Event
	endif
	if (cmpstr (eventKey, "kill") == 0)
		// Handle Kill Event
	endif
End Function

// ****************************************************
// button: close control panel
// ****************************************************

Function PPTCloseControl (ctrlName) : ButtonControl
	String	 ctrlName

	String topGraph = WinName (0, 1)
	SetWindow $topGraph, hook = $""

	KillControl pChooseAxis
	KillControl tMove
	KillControl bShiftLeft
	KillControl tMinValue
	KillControl tMaxValue
	KillControl bShiftRight
	KillControl cShowAxes
	KillControl bAutoScale
	KillControl bAutoScale_all
	KillControl bSetAllY
	KillControl bClose
	KillControl tDrawPos
	KillControl tDrawSize

	ControlBar 0
End Function

// ****************************************************
// popup: change current axis
// ****************************************************

Function PPTChooseAxis (ctrlName, popNum, popStr) : PopupMenuControl
	String		ctrlName
	Variable	popNum
	String		popStr

	String		currAxis

	NVAR		minVal	= root:PPTGlobals:pptMinVal
	NVAR		maxVal	= root:PPTGlobals:pptMaxVal

	currAxis = popStr
	GetAxis /Q $currAxis
	minVal = V_min
	maxVal = V_max

	if (PPTIsBottomOrTopAxis (currAxis))
		SetVariable tMinValue format="%.1f"
		SetVariable tMaxValue format="%.1f"
	else
		SetVariable tMinValue format="%.3g"
		SetVariable tMaxValue format="%.3g"
	endif

	PPTUpdateControlPanel ()
End Function

// ****************************************************
// checkbox: show axes instead of controlling waves
// ****************************************************

Function PPTUpdateAxesPopup (ctrlName, checked) : CheckBoxControl
	String		ctrlName
	Variable	checked
	NVAR		showAxes = root:PPTGlobals:pptShowAxes

	showAxes = checked
	PopupMenu pChooseAxis mode = 1
	PPTSetAxisPopupTitle ()
	ControlUpdate pChooseAxis
	PPTUpdateControlPanel ()
End Function

// ****************************************************

Function PPTSetAxisPopupTitle ()
	NVAR showAxes = root:PPTGlobals:pptShowAxes

	if (showAxes)
		PopupMenu pChooseAxis	title = "Axis"
	else
		PopupMenu pChooseAxis	title = "Wave"
	endif
End Function

// ****************************************************
// button: shift trace down/left or up/right
// ****************************************************

Function PPTShiftAxis (ctrlName) : ButtonControl
	String	 ctrlName

	NVAR minVal	= root:PPTGlobals:pptMinVal
	NVAR maxVal	= root:PPTGlobals:pptMaxVal
	NVAR delta		= root:PPTGlobals:pptDelta

	Variable realDelta = delta * (maxVal - minVal) / 100
	if (cmpstr (ctrlName, "bShiftLeft") == 0)
		realDelta *= -1
	endif
 
	minVal += realDelta
	maxVal += realDelta

	SetAxis $PPTGetCurrentAxis () minVal, maxVal
	ControlUpdate tMinValue
	ControlUpdate tMaxValue
End Function

// ****************************************************
// button: autoscales current axis
// ****************************************************

Function PPTAutoScaleAxis (ctrlName) : ButtonControl
	String	ctrlName

	String		currAxis
	Variable	V_min
	Variable	V_max

	NVAR minVal	= root:PPTGlobals:pptMinVal
	NVAR maxVal	= root:PPTGlobals:pptMaxVal

	currAxis = PPTGetCurrentAxis ()
	SetAxis /A $currAxis
	minVal = V_min
	maxVal = V_max

	PPTUpdateControlPanel ()
End Function

// ****************************************************
// autoscales all axes
// ****************************************************

Function PPTAutoScaleAll (ctrlName) : ButtonControl
	String ctrlName

	Variable	V_min
	Variable	V_max

	NVAR minVal	= root:PPTGlobals:pptMinVal
	NVAR maxVal	= root:PPTGlobals:pptMaxVal

	SetAxis /A
	minVal = V_min
	maxVal = V_max

	PPTUpdateControlPanel ()
End Function

// ****************************************************
// set all Y axes to min and max
// ****************************************************

Function PPTSetAllAxes (ctrlName) : ButtonControl
	String		ctrlName

	String		axisName
	String		theAxisList
	Variable	index = 0
	Variable	min, max

	// check if current axis is bottom or top axis
	if (PPTIsBottomOrTopAxis (PPTGetCurrentAxis ()))
		Abort "The current axis is a bottom/top axis, so the values won't fit to the left/right axes!"
	endif

	min = PPTGetVariableValue ("tMinValue")
	max = PPTGetVariableValue ("tMaxValue")

	theAxisList = AxisList ("")

	do
		// get the next axis name
		axisName = GetStrFromList (theAxisList, index, ";")
		if (strlen (axisName) == 0)
			break
		endif
		if (PPTIsBottomOrTopAxis (axisName) == 0)
			SetAxis $axisName min, max
		endif
		index += 1
	while (1)
End Function

// ****************************************************

Function PPTIsBottomOrTopAxis (theAxis)
	String	theAxis

	String	axInfo
	String	axType

	axInfo =  AxisInfo ("", theAxis)
	axType = StrByKey ("AXTYPE", axInfo)

	if ((cmpstr (axType, "bottom") == 0) %| (cmpstr (axType, "top") == 0))
		return (1)
	endif

	return (0)
End Function

// ****************************************************
// text-edit items: changes value for minimum and maximum
// ****************************************************

Function PPTSetMinMax (ctrlName, varNum, varStr, varName) : SetVariableControl
	String		ctrlName
	Variable	varNum
	String		varStr
	String		varName

	String		currAxis
	Variable	min, max

	currAxis = PPTGetCurrentAxis ()

	min = PPTGetVariableValue ("tMinValue")
	max = PPTGetVariableValue ("tMaxValue")
	SetAxis $currAxis min, max
End Function

// ****************************************************
// text-edit items: updates value for moving delta
// ****************************************************

Function PPTSetDelta (ctrlName, varNum, varStr, varName) : SetVariableControl
	String		ctrlName
	Variable	varNum
	String		varStr
	String		varName

	PPTUpdateControlPanel ()
End Function

// ****************************************************
// text-edit items: changes and updates value for minimum and maximum
// ****************************************************

Function PPTSetAxisMinMax (ctrlName, varNum, varStr, varName) : SetVariableControl
	String		ctrlName
	Variable	varNum
	String		varStr
	String		varName

	String		currAxis
	Variable	axisPos, axisSize

	NVAR pptAxisPos	= root:PPTGlobals:pptAxisPos
	NVAR pptAxisSize	= root:PPTGlobals:pptAxisSize

	axisPos = (PPTGetVariableValue ("tDrawPos")) / 100
	axisSize = (PPTGetVariableValue ("tDrawSize")) / 100
	axisPos = PPTSetValInRange (0, 1, axisPos)
	axisSize = PPTSetValInRange (0, 1 - axisPos, axisSize)

	if (pptAxisPos != (axisPos * 100))
		pptAxisPos = axisPos * 100
	endif
	if (pptAxisSize != (axisSize * 100))
		pptAxisSize = axisSize * 100
	endif
	currAxis = PPTGetCurrentAxis ()
	ModifyGraph axisEnab($currAxis)={axisPos, axisPos + axisSize}
End Function

// ****************************************************
// Background task: recalculates delta and updates controls
// ****************************************************

Function PPTUpdateControlPanel ()
	Variable	inc
	String		currAxis
	Variable	axisMin, axisMax
	Variable	V_min, V_max

	NVAR minVal		= root:PPTGlobals:pptMinVal
	NVAR maxVal		= root:PPTGlobals:pptMaxVal
	NVAR delta		= root:PPTGlobals:pptDelta
	NVAR axisPos		= root:PPTGlobals:pptAxisPos
	NVAR axisSize	= root:PPTGlobals:pptAxisSize

	currAxis = PPTGetCurrentAxis ()
	GetAxis /Q $currAxis

	minVal = V_min
	maxVal = V_max
	delta = PPTSetValInRange (0, 100, delta)
	inc = (maxVal - minVal) * delta / 100

	SetVariable tMinValue limits = {-INF,INF,inc}
	SetVariable tMaxValue limits = {-INF,INF,inc}

	String axInfo =  AxisInfo ("", currAxis)
	String axType = StrByKey ("AXTYPE", axInfo)

	axisMin = 100 * (GetNumFromModifyStr (axInfo, "axisEnab", "{", 0))
	axisMax = 100 * (GetNumFromModifyStr (axInfo, "axisEnab", "{", 1))
	axisPos = axisMin
	axisSize = axisMax - axisMin
End Function

// ****************************************************
// returns name of current axis
// ****************************************************

Function /S PPTGetCurrentAxis ()
	String		currAxis
	String		currWave

	String		theAxisList, theAxisName
	String		theList
	Variable	i
	String		S_Value

	NVAR showAxes	= root:PPTGlobals:pptShowAxes

	if (showAxes)
		ControlInfo pChooseAxis
		return (S_Value)
	endif

	ControlInfo pChooseAxis
	currWave = S_Value
	theAxisList = AxisList ("")
	i = 0
	do
		theAxisName = GetStrFromList (theAxisList, i, ";")
		if (strlen (theAxisName) == 0)					// end of list found
			return ("")
		endif

		if (cmpstr (currWave, StrByKey ("CWAVE", AxisInfo ("", theAxisName))) == 0)
			return (theAxisName)
		endif

		i += 1
	while (1)

	return ("")
End Function

// ****************************************************
// returns axis name, which is used by the given wave
// ****************************************************

Function /S PPTGetAxisOfTrace (waveName)
	String waveName
	String axisName, theString
	
	axisName = ""
	theString = TraceInfo ("", waveName, 0)
	if (strlen (theString) > 0)
		axisName = StrByKey ("YAXIS", theString)
	endif
		
	return (axisName)
End Function

// ****************************************************
// returns the actual value of a variable control
// ****************************************************

Function /D PPTGetVariableValue (ctrlName)
	String		ctrlName
	Variable	info
	Variable	value
	Variable	V_Value

	ControlInfo $ctrlName
	if ((V_Flag == 4) %| (V_Flag == 5))		// ValDisplay or SetVariable
		value = V_Value
	else
		value = 0
	endif

	return (value)
End Function

// ****************************************************

Function /D PPTSetValInRange (min, max, value)
	Variable /D min, max, value

	If (value < min)
		return (min)
	endif
	If (value > max)
		return (max)
	endif

	return (value)
End Function